@@ -1,6 +1,6 @@
 package ShardedKV::Storage::Redis::String;
 {
-  $ShardedKV::Storage::Redis::String::VERSION = '0.02';
+  $ShardedKV::Storage::Redis::String::VERSION = '0.03';
 }
 use Moose;
 # ABSTRACT: Storing simple string values in Redis
@@ -14,12 +14,12 @@ sub get {
   my $master = $self->redis_master;
   my $vref = \($master->get($key));
   Encode::_utf8_on($$vref); # FIXME wrong, wrong, wrong, but Redis.pm would otherwise call encode() all the time
-  return $vref;
+  return defined($$vref) ? $vref : undef;
 }
 
 sub set {
   my ($self, $key, $value_ref) = @_;
-  my $r = $self->master;
+  my $r = $self->redis_master;
   my $expire = $self->expiration_time;
   my $rv = $r->set($key, $$value_ref);
   $r->expire($key, $expire) if $expire;
@@ -39,7 +39,7 @@ ShardedKV::Storage::Redis::String - Storing simple string values in Redis
 
 =head1 VERSION
 
-version 0.02
+version 0.03
 
 =head1 SYNOPSIS
 
